@@ -93,6 +93,7 @@ static int flag_ksymoops = 1;
 static int n_ext_modules_used;
 static int m_has_modinfo;
 static int gplonly_seen;
+static int warnings;
 
 extern int insmod_main(int argc, char **argv);
 extern int insmod_main_32(int argc, char **argv);
@@ -1202,7 +1203,8 @@ static int init_module(const char *m_name, struct obj_file *f,
 		if (ret) {
 			error("init_module: %m");
 			lprintf("Hint: insmod errors can be caused by incorrect module parameters, "
-				"including invalid IO or IRQ parameters");
+				"including invalid IO or IRQ parameters.\n"
+			        "      You may find more information in syslog or the output from dmesg");
 		}
 	}
 
@@ -1331,6 +1333,7 @@ static int check_module_parameter(struct obj_file *f, char *key, char *value, in
 		 *        most error conditions.  Make these all errors in 2.5.
 		 */
 		lprintf("Warning: %s symbol for parameter %s not found", error_file, key);
+		++warnings;
 		return(1);
 	}
 
@@ -1345,6 +1348,7 @@ static int check_module_parameter(struct obj_file *f, char *key, char *value, in
 
 	if (max < min) {
 		lprintf("Warning: %s parameter %s has max < min!", error_file, key);
+		++warnings;
 		return(1);
 	}
 
@@ -1352,6 +1356,7 @@ static int check_module_parameter(struct obj_file *f, char *key, char *value, in
 	case 'c':
 		if (!isdigit(p[1])) {
 			lprintf("%s parameter %s has no size after 'c'!", error_file, key);
+			++warnings;
 			return(1);
 		}
 		while (isdigit(p[1]))
@@ -1365,9 +1370,11 @@ static int check_module_parameter(struct obj_file *f, char *key, char *value, in
 		break;
 	case '\0':
 		lprintf("%s parameter %s has no format character!", error_file, key);
+		++warnings;
 		return(1);
 	default:
 		lprintf("%s parameter %s has unknown format character '%c'", error_file, key, *p);
+		++warnings;
 		return(1);
 	}
 	switch (*++p) {
@@ -1382,6 +1389,7 @@ static int check_module_parameter(struct obj_file *f, char *key, char *value, in
 		break;
 	default:
 		lprintf("%s parameter %s has unknown format modifier '%c'", error_file, key, *p);
+		++warnings;
 		return(1);
 	}
 	return(0);
@@ -1435,10 +1443,16 @@ static void set_tainted(struct obj_file *f, int fd, int kernel_has_tainted,
 {
 	char buf[80];
 	int oldval;
+	static int first = 1;
 	if (fd < 0 && !kernel_has_tainted)
 		return;		/* New modutils on old kernel */
-	fprintf(stderr, "Warning: loading %s will taint the kernel: %s%s\n",
+	lprintf("Warning: loading %s will taint the kernel: %s%s",
 			f->filename, text1, text2);
+	++warnings;
+	if (first) {
+		lprintf("  See %s for information about tainted modules", TAINT_URL);
+		first = 0;
+	}
 	if (fd >= 0 && !noload) {
 		read(fd, buf, sizeof(buf)-1);
 		buf[sizeof(buf)-1] = '\0';
@@ -1683,8 +1697,10 @@ int INSMOD_MAIN(int argc, char **argv)
 	    (!persistdir || !*persistdir)) {
 		free(persist_name);
 		persist_name = NULL;
-		if (flag_verbose)
+		if (flag_verbose) {
 			lprintf("insmod: -e \"\" ignored, no persistdir");
+			++warnings;
+		}
 	}
 
 	if (m_name == NULL) {
@@ -1769,6 +1785,7 @@ int INSMOD_MAIN(int argc, char **argv)
 			      "\t%s was compiled for kernel version %s\n"
 				"\twhile this kernel is version %s",
 				filename, m_strversion, k_strversion);
+			++warnings;
 		} else {
 			if (!quiet)
 				error("kernel-module version mismatch\n"
@@ -2059,6 +2076,8 @@ int INSMOD_MAIN(int argc, char **argv)
 			delete_module(m_name);
 		goto out;
 	}
+	if (warnings && !noload)
+		lprintf("Module %s loaded, with warnings", m_name);
 	exit_status = 0;
 
       out:
